@@ -3,8 +3,7 @@ from deephys.deephys import (
     Layer,
     Model,
     model,
-    import_torch_dataset,
-    import_test_data,
+    test,
     ImageFile,
     Test,
     Stopwatch,
@@ -15,8 +14,7 @@ __all__ = [
     "Layer",
     "Model",
     "model",
-    "import_torch_dataset",
-    "import_test_data",
+    "test",
     "ImageFile",
     "Test",
     "Stopwatch",
